{---------------------------------------------------------------------------}
{ BitMachine Program:  PRCRANE                                              }
{ DATE:         04/11/99                                                    }
{ PROGRAMMER:   Terry King                                                  }
{ WHAT IT DOES: Runs Modified IBM 5152 Graphics Printer as CRANE or WINCH   }
{                                                                           }
{ HISTORY:                                                                  }
{ 02/07/99   Adding PF Key controls etc.                                    }
{ 02/08/99   Adding specific speed values to keys                           }
{ 02/14/99   Adding better screen display                                   }
{ 02/14/99   Controlling Motor Bits better between moves / startup          }
{ 02/14/99   Don't go below Zero on values!                                 }
{ 02/15/99   Allow smaller values near bottom of ranges, down to 1          }
{ 02/20/99   Adding control of magnet                                       }
{ 03/13/99   Adding Jog                                                     }
{ 03/18/99   Adding CROGO language loader / interpreter                     }
{ 03/24/99   Adding Learn Mode!!                                            }
{ 04/02/99   Break (F8) out of loop added                                   }
{ 04/05/99   Modify Key useage: Home/End Bouncebar, PGUP/PGDN INC/DEC values}
{ 04/11/99   Fix overwrite LL, change defaults speeds                       }
{                                                                           }
{---------------------------------------------------------------------------}

PROGRAM   PRCRANE ;

{---------------------------------------------------------------------------}
{               U S E S : WHAT OTHER PASCAL "UNITS" YOU WILL USE            }
{                                                                           }
{ NOTE: Other sets of tools or procedures that other people have created    }
{       that you want to re-use here.                                       }
{---------------------------------------------------------------------------}

USES
  CRT, DOS,                     { Standard Turbo Pascal UNITS }
  ParPort,                      { Parallel Port Support }
  Mwindow,                      { DOS Text windows}
  KbdInput,                     { ReadLnPF support for text and keystrokes}
  Sounds,                       { Sound effects}
  HexUtil;                      { Hex, Binary output }

{---------------------------------------------------------------------------}
{               C O N S T A N T S: FOR YOUR DESIGN                          }
{                                                                           }
{ NOTE: Here, you have to tell what is hooked up to the Digital In and      }
{       Digital Out Bits! Also other 'constants' for your design.           }
{---------------------------------------------------------------------------}
CONST

Version       = 'V1.50 [04/11/99: Fixes/Init] '; {++Program version}
CopyRight     = '(c) 1999 Mountain ClockWorks All Rights Reserved';
CopyRight2    = 'Educational / NonCommercial Use Permitted';
BLANKS        = '              ';


      {Program Window location}
PWIN_L_ULX      =  2; {Large window}
PWIN_L_ULY      =  6;
PWIN_S_ULX      =  2; {Small window}
PWIN_S_ULY      = 17;
PWIN_LRX        = 79;
PWIN_LRY        = 23;
STEPS2SHOW      = 16;

CHARCOLOR     = Yellow;   {Our screen colors}
BACKCOLOR     = Blue;
NOWAIT        = TRUE;      { for ReadLnPF }

APPEAR_NORM     = 1;   {Step line appearance: normal}
APPEAR_HIGH     = 2;   {HighLighted}
APPEAR_ERR      = 3;   {Error}

{-------( XY Locations for Display Values )---------------------------------}
DefaultX        = 78;
DefaultY        = 2 ;
MagnetX         = 54; {Magnet status}
MagnetY         = 12;

UDSS_X          = 54;  {Value displays}
UDSS_Y          = 5;
UDSM_X          = 54;
UDSM_Y          = 6;
RLSS_X          = 54;
RLSS_Y          = 7;
RLSM_X          = 54;
RLSM_Y          = 8;

PGMNAME_X        = 5;  {Program Name}
PGMNAME_Y        = 5;

STATE_X          = 63;  {RUN/STOPPED etc}
STATE_Y          = 14;

MSG_X            = 17;  {Message Box}
MSG_Y            = 24;

ACT_X            = 54;  { Action Status}
ACT_Y            = 10;

STEP_X           = 12;  { StepNow }
STEP_Y           = 24;

LOOP_X           = 54;  { Loop2Go }
LOOP_Y           = 16;

LEARN_X           = 54;  { LEARN }
LEARN_Y           = 18;

{-------( Learn Mode )------------------------------------------------------}
MAXSTEP       = 100;  {Max number of steps in StepArray }
MSGFIELDSIZE  = 43;
MAXMSGS       = 20;

ER            = 0; {error: Bad Code}
UD            = 1;
RL            = 2;
UP            = 3;
DN            = 4;
RT            = 5;
LF            = 6;
MG            = 7;
WA            = 8;
ST            = 9;
LP            = 10;

{-------( Motor Register Bits )--------------------------------------------}
UpDnMotorArray  : ARRAY[1..4] OF Byte = ($0C, $1C, $3C, $2C);
RtLfMotorArray  : ARRAY[1..4] OF Byte = ($40, $41, $43, $42);

MotorsHoldMask  = $4C;  {Pattern of Digital Out bits for low-power hold mode }
RLMotorMask     = $03;  { Bits that control motor phases }
UDMotorMask     = $30;
MagnetONMask    = $00;  {Bit 7 low is enable }
MagnetOFFMask   = $80;  {Bit 7 high is disable }

{-------( Digital IN Bits: What they are being used for:  )--- -------------}

FormFeedSw     =  $01;      { Switches on Printer Panel IF optional}
LineFeedSw     =  $02;      { Rewiring is done  }
PaperOutSw_    =  $04;
OnLineSw_      =  $20;
Pin_8B_1       =  $80;      { Uncommitted input pin: for WhatEver }

{-------( Digital OUT Bits: What they are being used for:  )------  }

                      {Left/Right (Printhead) Motor }
LRMotorA_B    = 0;    { Two Bits that control 4-step motor sequence }
LRMotorC_D    = 1;

_LRMotorEnab  = 2;    {- Enable 12 or 24V power to Left/Right Motor  }
_LRMotorHi    = 3;    {- 24V, + 12V  power: (PRCRANE Always uses 24V }

                      { Up/Down (Paper Feed) Motor}
UDMotorA_B    = 4;    { Two Bits that control 4-step motor sequence }
UDMotorC_D    = 5;

_UDMotorEnab  = 6;    {- Enable 24V power }

_MagnetBit    = 7;    { - enable optional Electromagnet connected from + 24V }


{---------------------------------------------------------------------------}
{               T Y P E S        : FOR YOUR DESIGN                          }
{                                                                           }
{ NOTE: Here, you define data structures called TYPEs for your design       }
{---------------------------------------------------------------------------}
TYPE
FileNameTYPE    = String[51];  {File Name Strings}
MaxString       = String[100];
MsgString       = String[MSGFIELDSIZE];

STRING8TYPE      = string[8];
STRING2TYPE      = string[2];
STEPTYPE         = string[2];      {Type of StepString data}

STEPENTRY =
    RECORD
      StepOper   : STRING2TYPE;  {Type of Operation: UD,RL,UP,DN,RT,LF,MG,WA,ST,LP}
      StepVal    : STRING8TYPE;  {Value for this step            }
    END;

BMSTATETYPE = (STOPPED, RUNNING, STEPPING, BREAKING, ENDED, ERRORSTOPPED);

{---------------------------------------------------------------------------}
{               V A R I A B L E S: FOR YOUR MACHINE DESIGN                  }
{                                                                           }
{ NOTE: Here, you have to tell what variables you will use to hold          }
{       temporary values for things. You can add lots more of these here.   }
{---------------------------------------------------------------------------}
VAR {MainLine}
    KeyByte     : Char;
    n           : integer;    { Integers are whole numbers up to 32,767 }
    Done        : Boolean;
    Debug       : Boolean;
    UpDateScreen: Boolean;
    QuietMODE   : Boolean;
    Appearance  : word;                    {Displayed line appearance}
    RtnCode     : word;

    ResponseType  : ReadlnPF_TYPE;           {For use by KbdInput Unit }
    KeyCode       : KeyCodeTYPE;
    UserString    : CommandStringTYPE;
    Statement     : CommandStringTYPE;

    EditPos       : Word;   {Bounce Bar edit Position: values}

    IndexUpDn     : Word;   {Stepper motor position index: global}
    IndexRtLf     : Word;

    RLMotorBits   : Byte;  { Bits of last RL Position }
    UDMotorBits   : Byte;  { Bits of last UD Position }
    MagnetBit     : Byte;  { Bit for Magnet ON/OFF }

    DelayMsUD     : Word;
    DelayMsRL     : Word;
    StartDelayMs  : Word;

    StepsUD       : Word;
    StepsRL       : Word;

    UDStepsSec    : Word;
    RLStepsSec    : Word;

    Xnow, Ynow    : Byte;                    {Cursor location}

{-------( Learn Mode Data )-------------------------------------------------}

    LearnMODE      : Boolean;
    StepArray      : ARRAY[1..MAXSTEP] OF STEPENTRY; {Array holding BitMachine Pgm}
    StepSave       : STEPENTRY;
    Oper           : Word;
    Value          : Word;
    LoopCount      : Word;
    Looping        : Boolean;
    LearnLines     : Word;

    MsgSaveArray   : ARRAY[1.. MAXMSGS] OF String[MSGFIELDSIZE];
    PgmDescription : String[50];              {Saved with file, shown at load time}


    PgmFile        : Text;                    {Program load/save}
    PgmFileName    : FileNameTYPE;
    OldPgmFileName : FileNameTYPE;

    EditorPath     : MaxString;               {Editor location}
    ParmStr        : MaxString;
    UtilStr        : MaxString;
    InWorkStr      : MaxString;               {Used in handling Input Data}
    InDataStr      : MaxString;
    OutWorkStr     : MaxString;               {Used in handling Output Data}
    OldOutStr      : MaxString;
    OutDataStr     : MaxString;

    Msg            : MsgString;

    KeyWord,
    KeyValue,
    KeyValue2,
    KeyValue3,
    Remainder      : MaxString;               {For #define, etc }

    BMState        : BMSTATETYPE;             {Current state of the BitMachine}

    KbChar           : char;                    { for kbd input }
    ChUtil           : char;
    Stop_Flag        : Boolean;                 {Error happened}

    Step             : word;               { Current Active BitMachine Step }
    StepLoopBack     : word;               { Step at beginning of this cycle}
    StepJumpTo       : word;               { Step to jump to via NextStep}
    Step2Show        : Integer;            { Step to show at top of window}
    PgmTopStep       : word;               { Top of program? }
    LastEdit         : word;               { Last edited line}

    MsgWritePtr      : word;               { Saved Msg}
    MsgReadPtr       : word;
    MsgMaxPtr        : word;               { Highest Saved Msg }

    ValueWord        : Word;               {Command Value}
    ValueWord2       : Word;
    ValueWord3       : Word;
    ValueLongInt     : longint;            {For bigger values}

(*Ŀ
   INTERNAL PROCEDURES and FUNCTIONS                                       
  *)


{------------------------------------------------------------------------}
{-----( ShowState PROCEDURE: )-------------------------------------------}
{------------------------------------------------------------------------}
PROCEDURE ShowState;

VAR Xnow, Ynow: Byte;
  BEGIN
    XNow := WhereX;
    YNow := WhereY;
    GoToXY(State_X, State_Y);
    IF (BMState = RUNNING)      THEN Write('RUNNING         ');
    IF (BMState = STOPPED)      THEN Write('STOPPED         ');
    IF (BMState = STEPPING)     THEN Write('STEPPING        ');
    IF (BMState = BREAKING)     THEN Write('BREAKING        ');
    IF (BMState = ENDED   )     THEN Write('PGM END         ');
    IF (BMState = ERRORSTOPPED) THEN Write('*ERROR*         ');
    GoToXY(Xnow, Ynow);
  END;


{------------------------------------------------------------------------}
{-----( BlankMsg PROCEDURE: Clear bottom status area             )-------}
{------------------------------------------------------------------------}
PROCEDURE BlankMsg;

VAR Xnow, Ynow: Byte;
  BEGIN
    XNow := WhereX;
    YNow := WhereY;
    GoToXY(MSG_X, MSG_Y);
    FOR N := 1 to MSGFIELDSIZE DO Write(' ');
    GoToXY(Xnow, Ynow);
  END;


{------------------------------------------------------------------------}
{-----( ShowMsg PROCEDURE: Write string to bottom status area    )-------}
{------------------------------------------------------------------------}
PROCEDURE ShowMsg(Msg:string);

VAR Xnow, Ynow: Byte;
  BEGIN
    XNow := WhereX;
    YNow := WhereY;
    GoToXY(MSG_X, MSG_Y);
    BlankMsg;
    GoToXY(MSG_X, MSG_Y);
    write(Msg);
(*  GoToXY(Xnow, Ynow);*)
  END;


{------------------------------------------------------------------------}
{-----( SaveMsg PROCEDURE: Write string to save array            )-------}
{------------------------------------------------------------------------}
PROCEDURE SaveMsg(Msg:string);

BEGIN
  INC(MsgWritePtr);
  IF (MsgWritePtr > MAXMSGS)
    THEN MsgWritePtr := 1
    ELSE MsgMaxPtr   := MsgWritePtr; {For retrieval}
  MsgSaveArray[MsgWritePtr] := Msg;
END;


{------------------------------------------------------------------------}
{-----( ShowMsgOpt PROCEDURE: Write string to bottom status area    )----}
{------------------------------------------------------------------------}
PROCEDURE ShowMsgOpt(Msg:string; Color: byte; Attr: byte);

VAR Xnow, Ynow: Byte;
  BEGIN
    XNow := WhereX;
    YNow := WhereY;
    GoToXY(MSG_X, MSG_Y);
    write('                                           ');
    GoToXY(MSG_X, MSG_Y);
    TextColor(Color + Attr);
    write(Msg);
    TextColor(CHARCOLOR);
(*  GoToXY(Xnow, Ynow);*)
  END;

{----------------------------------------------------------------}
{--( InitArrays: Initialize: at startup, before load )-----------}
{----------------------------------------------------------------}
PROCEDURE InitArrays;

BEGIN
  {-----( Initialize StepArray )---}
  FOR Step := 1 TO MAXSTEP DO
    BEGIN
      WITH StepEntry(StepArray[Step]) DO
        BEGIN
          StepOper   := 'ST';
          StepVal    := '        ';
        END;
    END; {Initialize StepArray}

  Step        := 1;
  Step2Show   := 1;
  PgmTopStep  := 1;


END;{InitArrays)



{----------------------------------------------------------------}
{--( InitScreen: Write the initial screen )----------------------}
{----------------------------------------------------------------}
PROCEDURE InitScreen;
BEGIN
  TextBackground(BACKCOLOR);
  TextColor(CHARCOLOR);
  ClrScr;

  Writeln('Ŀ');
  Writeln('HelpLoadQuitEditRun StopStepBrk LearnSaveP.R.CRANE                 ');
  Writeln(' F1  F2  F3  F4  F5  F6  F7  F8  F9   F10(c)1999 Mountain ClockWork');
  Writeln('Ĵ');
  Writeln('                                                                            ');
  Writeln('                                                                            ');
  Writeln('                                                                            ');
  Writeln('                                                                            ');
  Writeln('                                                  Ĵ');
  Writeln('                                                                            ');
  Writeln('                                                  Ĵ');
  Writeln('                                                                            ');
  Writeln('                                                  Ĵ');
  Writeln('                                                   PROGRAM:                 ');
  Writeln('                                                  Ĵ');
  Writeln('                                                                            ');
  Writeln('                                                  Ĵ');
  Writeln('                                                                            ');
  Writeln('                                                  Ĵ');
  Writeln('                                                                            ');
  Writeln('                                                  Ĵ');
  Writeln('                                                                            ');
  Writeln('Ĵ');
  Writeln('StepNow = 000                                                              ');
  Write  ('');

  GoToXY(2,17);
  ShowMsg('P.R.CRANE  V: ' + Version );
END;{InitScreen}


{----------------------------------------------------------------}
{--( GetUserString  : Prompt, get string from user  )------------}
{----------------------------------------------------------------}
FUNCTION GetUserString(PromptStr : MaxString;
                       DefaultStr: MaxString;
                       Xloc      : Byte;
                       Yloc      : Byte)
                                 : MaxString;
VAR

Xnow, Ynow: Byte;
FieldSize : word;

BEGIN
  Xnow := WhereX;
  Ynow := WhereY;
  FieldSize := Length(PromptStr) + Length(DefaultStr) + 5;
  GoToXY(Xloc, Yloc);
  BlankMsg;
  GoToXY(Xloc, Yloc);
  Write(PromptStr,DefaultStr);
  ResponseType := ReadlnPF(UserString, KeyCode, FALSE );{Get user input}
  IF (ResponseType = Esc) THEN ;   {??}
  IF (ResponseType = CommandStr ) OR
     (ResponseType = AlphaNumChar)
    THEN
      BEGIN
        IF (Length(UserString) > 0)
          THEN GetUserString := UserString
          ELSE GetUserString := DefaultStr;
      END;

  {--( Blank out where we were )----}
  GoToXY(Xloc, Yloc);
  BlankMsg;

  GoToXY(Xnow,Ynow);
END;{GetUserString}

{----------------------------------------------------------------}
{--( CleanLeadBlanks  : Remove leading blanks from a string )----}
{----------------------------------------------------------------}
PROCEDURE CleanLeadBlanks(VAR InStr: MaxString);

BEGIN
  WHILE (InStr[1] = ' ') AND (Length(InStr) > 0)
    DO Delete(InStr,1,1);
END;


{----------------------------------------------------------------}
{--( PadBlanksTo  : Pad with trailing blanks to desired length)--}
{----------------------------------------------------------------}
PROCEDURE PadBlanksTo(VAR InStr: MaxString; PadLength: word);

BEGIN
  WHILE (Length(InStr) < PadLength) DO InStr := Instr + ' ';
END;


{----------------------------------------------------------------}
{--( GetNext Procedure: Get next token )-------------------------}
{  InString : Input String with one or more tokens               }
{  Token    : Result String with first token found               }
{  RemainStr: String containing the rest of InString after Token }
{----------------------------------------------------------------}
Procedure GetNext(InString: maxstring;
                        ch: char;
                 Var token: maxstring;
            Var  RemainStr: maxstring);
Var
  SepLocation :integer;   {separator position}

BEGIN
  SepLocation := pos(ch,InString);                  {Locate 1st separator char}
  IF SepLocation <> 0 THEN                            {IF separator char found}
  BEGIN
    token := copy(InString,1,SepLocation);                      {Extract Token}
    delete(token,SepLocation,1);                   {Remove separator character}
    RemainStr := InString;
    delete(RemainStr,1,SepLocation);                 {Delete preseparator text}
    WHILE     (RemainStr[1]=ch)             {Remove additional separator chars}
          AND (length(RemainStr)>0)
      DO delete(RemainStr,1,1);
  END                                                       {A token was found}
  ELSE                                                {No separator char found}
  BEGIN
    token := InString;                           {Put entire string into token}
    RemainStr := '';                                        {RemainStr is null}
  END;
END;


{------------------------------------------------------------------}
{-----( ERROR PROCEDURE:  Print error and HALT )-------------------}
{------------------------------------------------------------------}
  procedure Error(msg: string);
    begin
      WriteLn('ERROR IN EXTERNAL PROGRAM: ' + msg);
      halt;

    end;

{------------------------------------------------------------------}
{    EXECANDCHECK PROCEDURE:  Execute an external program          }
{                                                                  }
{  NOTE: The MAIN program used MUST provide free memory for program}
{        to be execute by setting 'Maximum Heap Size' to a small   }
{        but useable value (say 10000)                             }
{                                                                  }
{------------------------------------------------------------------}

  procedure ExecAndCheck(Pgm: PathStr; CmdLine: string);
    begin
      SwapVectors;
      Exec(Pgm,CmdLine);
      SwapVectors;
      case DosError of
         2: Error('File not found: '+Pgm);
         8: Error('Insufficient memory to run '+Pgm);
        10: Error('Invalid environment');
        11: Error('Invalid format');
      end;
    end;


{------------------------------------------------------------------------}
{-----( CALL PROCEDURE: Execute external program by it's name only )-----}
{------------------------------------------------------------------------}

  procedure Call(PgmName: PathStr; CmdLine: ComStr; var ErrorLevel: word);

  VAR   PgmPath   : PATHSTR;
        PgmEnvPath : string;
    begin
      PgmEnvPath := GetEnv('PATH');
      PgmPath := FSearch(PgmName,PgmEnvPath);
      IF (Length(PgmPath) <> 0) THEN ExecAndCheck(PgmPath,CmdLine);
      ErrorLevel := DosExitCode;
    end;



{------------------------------------------------------------------------}
{-----( AddStepEntry: Put a StepEntry in StepArray  )--------------------}
{------------------------------------------------------------------------}

PROCEDURE AddStepEntry(StepPos : word;
                       Oper    : STRING2TYPE;
                       Value   : STRING8TYPE);

BEGIN
  WITH StepEntry(StepArray[StepPos]) DO
    BEGIN
      StepOper   := Oper;
      StepVal    := Value;
    END;
END;



{------------------------------------------------------------------------}
{-----( ShowStepEntry: Write a StepEntry on the display)-----------------}
{------------------------------------------------------------------------}

PROCEDURE ShowStepEntry(StepPos : word; Appearance: word);

VAR ValStr: String[4];
    N     : word;

BEGIN
  WITH StepEntry(StepArray[StepPos]) DO
    BEGIN
      IF (Appearance = APPEAR_HIGH ) THEN TextBackGround(LightCyan);
      IF (Appearance = APPEAR_ERR  ) THEN TextBackGround(RED);
      Str(StepPos:4, ValStr);

      Write( ValStr + '  ', StepOper, '  ', StepVal:5, '    ');     {Stepname and Step}

    END;

  TextBackground(BACKCOLOR);
END;


{------------------------------------------------------------------------}
{--( LoadPgm    PROCEDURE: Load a BitMachine Program from File   )-------}
{------------------------------------------------------------------------}
Procedure LoadPgm(FileName: FileNameTYPE);

VAR
StepString  : MaxString;
ValueStr    : STRING8TYPE;
OperStr     : STRING2TYPE;

NumPgmLines    : word;    {Actual program lines}
NumLines       : word;    {Total ALL lines}

IsDefineLine   : Boolean;

BEGIN
  Stop_Flag := FALSE;
  Debug := FALSE;
  Assign(PgmFile,Filename);
  {$I-} Reset(PgmFile) {$I+};
  IF (IOresult<>0) THEN
    BEGIN
      Msg := '**ERROR**  Cannot find file ' + Filename;
      ShowMsgOpt(Msg,LightRed,0);
      SaveMsg(Msg);
      Stop_Flag := true;
    END
  ELSE {It's Cool: Go ahead}
    BEGIN
      TextColor(LightRed);
      GoToXY(PGMNAME_X, PGMNAME_Y);
      Write('            ');
      GoToXY(PGMNAME_X, PGMNAME_Y);
      Write(Filename);
      TextColor(CHARCOLOR);

      IF Debug THEN WriteLn;
      NumPgmLines := 0;
      IF NOT Stop_Flag THEN
        BEGIN
          WHILE (NOT Eof(PgmFile)) AND (NOT Stop_Flag)  DO
          BEGIN {Processing File Lines}

            Readln(PgmFile,StepString);      {Read statement}
            NumLines := NumLines + 1;

            {------( Check for #define line )---------}
            IF ( Copy(StepString, 1, 8) = '#define ' )
            THEN
              BEGIN
                IsDefineLine := TRUE;
                KeyWord  := '';
                KeyValue := '';
                {---( Delete #define )----}
                Delete(StepString,1,8);

                CleanLeadBlanks(StepString);
                GetNext(StepString, ' ', KeyWord, Remainder);

                {---------( Have a KeyWord. What to do?? )--------}
           (*     ShowMsg(KeyWord);*)

       {----( PgmDescription Keyword )--------------------}
                IF (KeyWord = 'PgmDesc') THEN
                  BEGIN
                    PgmDescription := Remainder;
                    Msg := 'PgmDesc: ' + PgmDescription;
                    ShowMsg(Msg);
                    SaveMsg(Msg);
                  END;

        {--------------}
              END{Handle #define}
            ELSE
              BEGIN
                IsDefineLine := FALSE;
              END;
  {---------------( Load program steps )--------------------------}

            IF NOT IsDefineLine THEN
              BEGIN   {------( Handle A Normal Line )-----------}
                IF   ( StepString[3] = ' ')
                THEN
                  BEGIN
                    NumPgmLines := NumPgmLines + 1;
                  {-------( Extract Strings from StepString )---}
                    OperStr        := Copy(StepString, 1, 2);
                    ValueStr       := Copy(StepString, 4, 8);

                    IF Debug THEN
                      WriteLn(OperStr+' '+ValueStr);

                    AddStepEntry(NumPgmLines,
                                 OperStr,
                                 ValueStr);
                  END
                ELSE
                  BEGIN
                    Str((NumPgmLines + 1):3, InWorkStr );
                    UtilStr :='!!ERROR!!  On Line # ' + InWorkStr;
                    ShowMsgOpt(UtilStr,LightRed,0);
                    Delay(3000);
                  END;
              END;

            UpDateScreen := TRUE;
            PgmTopStep   := NumPgmLines;
            Step         := 1; {??}
  {---------------( END: Load program steps )--------------------------}


          END;{Processing Load File Lines}
        IF NOT Stop_Flag THEN Close(PgmFile);
       END;{NOT Stop_Flag}
    END;
END;{LoadPgm}



{------------------------------------------------------------------------}
{--( SavePgm    PROCEDURE: Save a BitMachine Program to   File   )-------}
{------------------------------------------------------------------------}
Procedure SavePgm(FileName: FileNameTYPE);

VAR
PgmLine     : MaxString;
StepString  : MaxString;
ValueStr    : STRING8TYPE;
OperStr     : STRING2TYPE;

NumLines    : word;

BEGIN
  Debug := FALSE;
  Assign(PgmFile,Filename);
  {$I-} Rewrite(PgmFile) {$I+};
  IF (IOresult<>0) THEN
    BEGIN
      Msg := '**ERROR**  Cannot open file ' + Filename;
      ShowMsgOpt(Msg, LightRed, 0);
      SaveMsg(Msg);
      Stop_Flag := true;
    END
  ELSE {It's Cool: Go ahead}
    BEGIN
      IF Debug THEN WriteLn;

   {-------( Write out #defines for IN/OUT/Variables    )--------}

      {----( Write the PgmDesc definition )----}
      PgmLine := '#define ' + 'PgmDesc ' + PgmDescription;
      WriteLn(PgmFile,PgmLine);

   {-------( Write out the Program lines from StepArray )--------}
      NumLines := PgmTopStep;

      FOR n := 1 to NumLines DO
        BEGIN {Processing File Lines}

        {-------( Read out of Array )---}
        WITH StepArray[n] DO
          BEGIN

        {-------( Build Output Line  )---}
            OperStr  := StepOper;
            ValueStr := StepVal;
            PgmLine    := OperStr     +
                          ' '         +
                          ValueStr    ;

            WriteLn(PgmFile,PgmLine);
          END;

        END;{Processing Save File Lines}
      Close(PgmFile);
    END;{Its OK}
END;{SavePgm}

{-----------------------------------------------------------------------}

PROCEDURE MotorsHold;

VAR MotorsHoldData : Byte;

BEGIN
  RLMotorBits     := (RtLfMotorArray[IndexRtLf] AND RLMotorMask);
  UDMotorBits     := (UpDnMotorArray[IndexUpDn] AND UDMotorMask);
  MotorsHoldData  := MotorsHoldMask OR RLMotorBits OR UDMotorBits OR MagnetBit;
  WriteDo(MotorsHoldData);
  IF Debug THEN Writeln('MotorsHOLD: ', Byte2Bin(MotorsHoldData));
END;{MotorsHold}

{-----------------------------------------------------------------------}

PROCEDURE MagnetON;

BEGIN
  MagnetBit := MagnetOnMASK;
  MotorsHold;
  GoToXY(MagnetX, MagnetY);
  TextColor(LightRed);
  Write('MAGNET ON    ');
  TextColor(CHARCOLOR);
  {---( Learn Mode )------}
  IF LearnMODE THEN
    BEGIN
      AddStepEntry(LearnLines,
                   'MG',
                   '1');
      INC(LearnLines);
    END;
END;

{-----------------------------------------------------------------------}

PROCEDURE MagnetOFF;

BEGIN
  MagnetBit := MagnetOffMASK;
  MotorsHold;
  GoToXY(MagnetX, MagnetY);
  TextColor(LightGreen);
  Write('MAGNET OFF   ');
  TextColor(CHARCOLOR);

  IF LearnMODE THEN
    BEGIN
      AddStepEntry(LearnLines,
                   'MG',
                   '0');
      INC(LearnLines);
    END;
END;

{-----------------------------------------------------------------------}

PROCEDURE ShowAction(Action: String);
VAR SaveX, SaveY : Word;

  BEGIN
    SaveX := WhereX;
    SaveY := WhereY;
    GoToXY(ACT_X, ACT_Y);
    Write(Action);
(*  GoToXY(SaveX, SaveY);*)
  END;

{-----------------------------------------------------------------------}
Procedure CraneUp(Steps: Word; StepDelayMs: Word);
{NOTE: Uses Global IndexUpDn }

BEGIN
  N := 0;
  ShowAction('MOVING UP');
  WriteDO(UpDnMotorArray[IndexUpDn]
          OR RLMotorBits
          OR MagnetBit); {Power up with last Pattern }
  Delay(StartDelayMs);
  REPEAT
    BEGIN
      INC(n);
      IF (IndexUpDn = 4) THEN IndexUpDn := 1 ELSE INC(IndexUpDn);
      WriteDO(UpDnMotorArray[IndexUpDn]
              OR RLMotorBits
              OR MagnetBit);
      IF Debug THEN Writeln('UP: ', Byte2Bin(UpDnMotorArray[IndexUpDn]
                            OR RLMotorBits
                            OR MagnetBit));
      Delay(StepDelayMs);
    END;
  UNTIL (n = Steps)  OR KeyPressed;
  UDMotorBits := (UpDnMotorArray[IndexUpDn] AND UDMotorMask);
  MotorsHold;
  ShowAction(BLANKS);

  IF LearnMODE THEN
    BEGIN
      Str(n:8, UtilStr);
      AddStepEntry(LearnLines,
                   'UP',
                   UtilStr);
      INC(LearnLines);
    END;

END;{CraneUp}

{-----------------------------------------------------------------------}

Procedure CraneDn(Steps: Word; StepDelayMs: Word);
{NOTE: Uses Global IndexUpDn }

BEGIN

  N := 0;
  ShowAction('MOVING DOWN');
  WriteDO(UpDnMotorArray[IndexUpDn]
          OR RLMotorBits
          OR MagnetBit); {Power up with last Pattern }
  Delay(StartDelayMs);
  REPEAT
    BEGIN
      INC(n);
      IF (IndexUpDn = 1) THEN IndexUpDn := 4 ELSE DEC(IndexUpDn);
      WriteDO(UpDnMotorArray[IndexUpDn]
              OR RLMotorBits
              OR MagnetBit);
      IF Debug THEN Writeln('DN: ', Byte2Bin(UpDnMotorArray[IndexUpDn]
                                             OR RLMotorBits
                                             OR MagnetBit));
      Delay(StepDelayMs);
    END;
  UNTIL (n = Steps)  OR KeyPressed;
  MotorsHold;
  ShowAction(BLANKS);

  IF LearnMODE THEN
    BEGIN
      Str(n:8, UtilStr);
      AddStepEntry(LearnLines,
                   'DN',
                   UtilStr);
      INC(LearnLines);
    END;

END;{CraneDn}

{-----------------------------------------------------------------------}

Procedure CraneLf(Steps: Word; StepDelayMs: Word);
{NOTE: Uses Global IndexRtLf }

BEGIN

  N := 0;
  ShowAction('MOVING LEFT');
  WriteDO(RtLfMotorArray[IndexRtLf]
          OR UDMotorBits
          OR MagnetBit); {Power up with last Pattern }
  Delay(StartDelayMs);
  REPEAT
    BEGIN
      INC(n);
      IF (IndexRtLf = 4) THEN IndexRtLf := 1 ELSE INC(IndexRtLf);
      WriteDO(RtLfMotorArray[IndexRtLf]
                             OR UDMotorBits
                             OR MagnetBit);
      IF Debug THEN Writeln('LF: ', Byte2Bin(RtLfMotorArray[IndexRtLf]
                                             OR UDMotorBits
                                             OR MagnetBit));
      Delay(StepDelayMs);
    END;
  UNTIL (n = Steps) OR KeyPressed;
  MotorsHold;
  ShowAction(BLANKS);
  IF LearnMODE THEN
    BEGIN
      Str(n:8, UtilStr);
      AddStepEntry(LearnLines,
                   'LF',
                   UtilStr);
      INC(LearnLines);
    END;

END;{CraneLf}

{-----------------------------------------------------------------------}

Procedure CraneRt(Steps: Word; StepDelayMs: Word);
{NOTE: Uses Global IndexRtLf }

BEGIN

  N := 0;
  ShowAction('MOVING RIGHT');
  WriteDO(RtLfMotorArray[IndexRtLf]
          OR UDMotorBits
          OR MagnetBit); {Power up with last Pattern }
  Delay(StartDelayMs);
  REPEAT
    BEGIN
      INC(n);
      IF (IndexRtLf = 1) THEN IndexRtLf := 4 ELSE DEC(IndexRtLf);
      WriteDO(RtLfMotorArray[IndexRtLf]
              OR UDMotorBits
              OR MagnetBit);
      IF Debug THEN Writeln('RT: ', Byte2Bin(RtLfMotorArray[IndexRtLf]
                                             OR UDMotorBits
                                             OR MagnetBit));
      Delay(StepDelayMs);
    END;
  UNTIL (n = Steps) OR KeyPressed;
  MotorsHold;
  ShowAction(BLANKS);
  IF LearnMODE THEN
    BEGIN
      Str(n:8, UtilStr);
      AddStepEntry(LearnLines,
                   'RT',
                   UtilStr);
      INC(LearnLines);
    END;

END;{CraneRt}


{------------------------------------------------------------------------}
{-----( ExecuteStep PROCEDURE: Do the current Step(s)   )----------------}
{------------------------------------------------------------------------}
PROCEDURE ExecuteStep;

  BEGIN
    WITH StepArray[Step] DO
      BEGIN
        Oper := ER;                          {Init as ERROR}
        IF StepOper = 'UD' THEN Oper := UD;
        IF StepOper = 'RL' THEN Oper := RL;
        IF StepOper = 'UP' THEN Oper := UP;
        IF StepOper = 'DN' THEN Oper := DN;
        IF StepOper = 'RT' THEN Oper := RT;
        IF StepOper = 'LF' THEN Oper := LF;
        IF StepOper = 'MG' THEN Oper := MG;
        IF StepOper = 'WA' THEN Oper := WA;
        IF StepOper = 'ST' THEN Oper := ST;
        IF StepOper = 'LP' THEN Oper := LP;
        Val(StepVal, Value, RtnCode);
        IF (RtnCode <> 0 ) THEN
          BEGIN
            ShowMsgOpt('Value Error on current step!',LightRed,0);
            Delay(3000);
          END;
        CASE Oper OF
          ER:   {ERROR: Bad Code}
            BEGIN
              ShowMsgOpt('BAD / UNRECOGNIZED CODE!',LightRed,0);
              Delay(3000);
            END;{'UD'}

          UD:   {Up/Down Delay Ms}
            BEGIN
              DelayMsUD := Trunc((1.0 / Value) * 1000);
            END;{'UD'}

          RL:   {Rt/Left Delay Ms}
            BEGIN
              DelayMsRL := Trunc((1.0 / Value) * 1000);
             END;{'RL'}

          UP:   {Up Steps}
            BEGIN
              CraneUp( Value, DelayMsUD);
            END;{'UP'}

          DN:   {Dn Steps}
            BEGIN
              CraneDn( Value, DelayMsUD);
            END;{'DN'}

          RT:   {RT Steps}
            BEGIN
              CraneRt( Value, DelayMsRL);
            END;{'RT'}

          LF:   {LF Steps}
            BEGIN
              CraneLf( Value, DelayMsRL);
            END;{'LF'}

          MG:   {Magnet}
            BEGIN
              IF (Value = 1) THEN MagnetOn ELSE MagnetOff;
            END;{'MG'}

          WA:   {Wait}
            BEGIN
              GoToXY(State_X, State_Y);
              Write('WAITING ', Value:5, ' Ms');
              Delay(Value);
            END;{'WA'}

          ST:   {Stop}
            BEGIN
              BMState := STOPPED;
            END;{'ST'}

          LP:   {Loop}
            BEGIN
              IF NOT Looping
                THEN
                  BEGIN
                    LoopCount := (Value -1);
                    GoToXY(LOOP_X,LOOP_Y);
                    Write('Loops To Go: ',LoopCount:3);
                    Looping := TRUE;
                  END
                ELSE
                  BEGIN
                    DEC(LoopCount);
                    GoToXY(LOOP_X,LOOP_Y);
                    Write('Loops To Go: ',LoopCount:3);

                    IF (LoopCount <= 0)
                    THEN
                      BEGIN
                        BMState := ENDED;
                        Looping := FALSE;
                      END;
                  END;

              IF    (BMState = RUNNING)
                 OR (BMState = STEPPING)
              THEN
                BEGIN
                  Step := 1; {??}
                END;
            END;{'LP'}

        END;{CASE Oper}

      END;{WITH}
      INC(Step);
      Step2Show := Step;

      UpdateScreen := TRUE;

      IF (BMState = STEPPING)     THEN BMState := STOPPED;
      IF (Step > PgmTopStep)      THEN BMState := ENDED;
      IF (BMState = ENDED)
        THEN
          BEGIN
            Step      := 1;
            Step2Show := 1;
          END;
  END;{ExecuteStep}


(*Ŀ
   MAIN PROGRAM:                                                           
  *)
BEGIN

  {---------( INITIALIZE )-----------------------------------------}
  BMState          := STOPPED;
  Step             := 1;

  MsgWritePtr      := 0;    {Saved messages}
  MsgReadPtr       := 1;
  MsgMaxPtr        := 1;
  PgmFileName      := '';
  OldPgmFileName   := 'DEFAULT.CRL';

  EditPos          := 1;    {BounceBar: Edit values}
  LearnLines       := 1;

  InitArrays;

  InitScreen;

  delay(200);

  UpDateScreen := TRUE;

{-----( Check environment variable EDITOR for editor name/path )-----------}

  ParmStr := GetEnv('EDITOR');
  IF (length(ParmStr) > 0) THEN
    BEGIN
(*      Writeln('ENVIRONMENT VARIABLE: EDITOR = ', ParmStr);*)
      EditorPath := ParmStr;
    END;

  {---------( TRY A CALL )-----------------------------------------}
{??}
 {---------( INITIALIZE: SET THINGS UP AT THE START )-----------------------}

 Debug      := FALSE;
 Done       := FALSE;
 Looping    := FALSE;

 IndexUpDn    := 1;       {Initialize so initial motor bit pattern is known}
 IndexRtLf    := 1;
 DelayMsUD    := 2;
 DelayMsRL    := 50;
 StepsUD      := 2000;
 StepsRL      := 100;
 StartDelayMs := 100;
 RLMotorBits  := 0;
 UDMotorBits  := 0;
 MagnetBit    := MagnetOffMASK;
 MotorsHold;         { Set motor, magnet bits to low-power hold mode}

  TextBackGround(BACKCOLOR);
  TextColor(CHARCOLOR);
(* ClrScr; *)
(*
  GoToXY(1,1);
  Writeln('             ----   P.R.CRANE: Rewired IBM Graphics Printer   ---   ');
  writeln('                     ',version);
  writeln('              ',Copyright);
  writeln('                     ',Copyright2);
  writeln('     Check for latest Information: http://homepages.together.net/~tking');
  GoToXY(1,24);
  Write('F3 to EXIT     <Ctrl> AND <Break> TOGETHER TO INTERRUPT A MOVE');
  GoToXY(1,7);
  Write('         KEYBOARD ARROW KEYS MOVE UP / DOWN / LEFT / RIGHT     ');
*)
  UpDateScreen := TRUE;

{---( MAIN LOOP IN PROGRAM: THIS PART RUNS OVER AND OVER AGAIN )------------}
  REPEAT

    ShowState;
    IF ((BMState = RUNNING) OR (BMState = STEPPING))
      THEN
        BEGIN
          IF NOT QuietMODE THEN Click;
          ExecuteStep;
        END;
    ShowState;


    IF UpDateScreen THEN
      BEGIN
      {-------( UPDATE PROGRAM DISPLAY SCREEN )-------- }
        Window(PWIN_L_ULX, PWIN_L_ULY, PWIN_LRX, PWIN_LRY);
        FOR n := Step2Show TO (Step2Show + STEPS2SHOW) DO
        BEGIN
          IF (n = Step2Show )
            THEN Appearance := APPEAR_HIGH
            ELSE Appearance := APPEAR_NORM;
          ShowStepEntry(n , Appearance); Writeln;
        END;
        Window(1,1,80,25);
        UpDateScreen := FALSE;

   {---------( SHOW STEPS/SEC, MOVES ETC )--------------------------}
        UDStepsSec   :=  1000 DIV DelayMsUD;
        RLStepsSec   :=  1000 DIV DelayMsRL;

        GoToXY(UDSS_X,UDSS_Y);
        IF EditPos = 1 THEN TextColor(LightGreen);
        Write('UD ', UDStepsSec:4, ' STEPS/SEC        ');
        TextColor(CHARCOLOR);

        GoToXY(UDSM_X,UDSM_Y);
        IF EditPos = 2 THEN TextColor(LightGreen);
        Write('UD ', StepsUD:4   , ' STEPS/MOVE       ');
        TextColor(CHARCOLOR);

        IF EditPos = 3 THEN TextColor(LightGreen);
        GoToXY(RLSS_X,RLSS_Y);
        Write('RL ', RLStepsSec:4, ' STEPS/SEC        ');
        TextColor(CHARCOLOR);

        IF EditPos = 4 THEN TextColor(LightGreen);
        GoToXY(RLSM_X,RLSM_Y);
        Write('RL ', StepsRL:4   , ' STEPS/MOVE       ');
        TextColor(CHARCOLOR);

        GoToXY(STEP_X,STEP_Y);
        Write(Step:3);
        GoToXY(DefaultX,DefaultY);

      END;{Update Screen}

   {---------( GET USER INPUT, CHECK, HANDLE DIFFERENT TYPES )------}
    UpDateScreen := FALSE;
    UserString := '';
    KeyCode    := 0;
    ResponseType := ReadlnPF(UserString, KeyCode, NOWAIT );  {Get user input}
    IF (ResponseType <> NoKey) THEN
      BEGIN
        IF (ResponseType = CommandStr) THEN
          BEGIN
            Statement := UserString;
            UserString := '';
          END;
      {------------( Look for ESC Key )-------------------------}

        IF (ResponseType = Esc) THEN ;

      {------------( Look for Extended Keys )-------------------------}

        IF (ResponseType = ExtendedKey) THEN
          BEGIN
            UpDateScreen := TRUE;

{-------------------( USER CONTROL OF MOVEMENTS )-----------------------}
            IF (KeyCode = UPARROW)    THEN CraneUp(StepsUD, DelayMsUD);

            IF (KeyCode = DOWNARROW)  THEN CraneDn(StepsUD, DelayMsUD);

            IF (KeyCode = LEFTARROW)  THEN CraneLf(StepsRL, DelayMsRL);

            IF (KeyCode = RIGHTARROW) THEN CraneRt(StepsRL, DelayMsRL);


            IF (KeyCode = INS      ) THEN MagnetON;
            IF (KeyCode = DEL      ) THEN MagnetOff;

{-------------------( Control Edit Bounce Bar )--------------------------}
            IF (KeyCode = _END ) THEN
              BEGIN
                IF (EditPos < 4)
                  THEN INC(EditPos)
                ELSE EditPos := 1;
              END;
            IF (KeyCode =  HOME    ) THEN
              BEGIN
                IF (EditPos > 1)
                  THEN DEC(EditPos)
                ELSE EditPos := 4;
              END;

{-------------------( USER EDIT VALUES OF STEPS / SPEED )---------------}

            IF (KeyCode = PGUP   ) THEN
              BEGIN
                IF (EditPos = 1) THEN
                  BEGIN
                    IF DelayMsUD < 6
                      THEN DelayMsUD := 4
                      ELSE DelayMsUD := DelayMsUD - (DelayMsUD DIV 2);
                  END;
                IF (EditPos = 2) THEN
                  BEGIN
                    IF StepsUD >= 50
                      THEN INC(StepsUD, 25)
                      ELSE INC(StepsUD, 1);
                  END;
                IF (EditPos = 3) THEN
                  BEGIN
                    IF DelayMsRL < 6
                      THEN DelayMsRL := 4
                      ELSE DelayMsRL := DelayMsRL - (DelayMsRL DIV 2);
                  END;
                IF (EditPos = 4) THEN
                  BEGIN
                    IF StepsRL >= 50
                      THEN INC(StepsRL, 25)
                      ELSE INC(StepsRL, 1);
                  END;
              END;{PgUp}

            IF (KeyCode = PGDN ) THEN
              BEGIN
                IF (EditPos = 1) THEN
                  BEGIN
                    IF DelayMsUD < 4
                      THEN DelayMsUD := 4
                      ELSE DelayMsUD := DelayMsUD - (DelayMsUD DIV 2);
                  END;
                IF (EditPos = 2) THEN
                  BEGIN
                    IF StepsUD >= 50
                      THEN DEC(StepsUD, 25)
                      ELSE DEC(StepsUD, 1);
                  END;
                IF (EditPos = 3) THEN
                  BEGIN
                    IF DelayMsRL < 6
                      THEN DelayMsRL := 6
                      ELSE DelayMsRL := DelayMsRL + (DelayMsRL DIV 2);
                  END;
                IF (EditPos = 4) THEN
                  BEGIN
                    IF StepsRL >= 50
                      THEN DEC(StepsRL, 25)
                      ELSE DEC(StepsRL, 1);
                  END;

              END;{MINUS}


            IF (KeyCode = ALT_D    ) THEN
              BEGIN
                BlankMsg;
                PgmDescription := GetUserString('PgmDesc= ','',17,24);
                ShowMsg('PgmDesc: ' + PgmDescription);
              END;

            IF NOT LearnMODE THEN
              BEGIN
                IF (KeyCode = ALT_R    ) THEN
                  BEGIN
                    ShowMsg('RESTART');
                    Step := 1;
                    Step2Show := 1;
                    UpDateScreen := TRUE;
                  END;

                IF (KeyCode = ALT_N    ) THEN
                  BEGIN
                    ShowMsg('NEW PGM');
                    InitArrays;
                    PgmFileName    := 'NEWPGM.CRL';
                    OldPgmFileName := 'NEWPGM.CRL';
                    TextColor(LightRed);
                    GoToXY(PGMNAME_X, PGMNAME_Y);
                    Write('            ');
                    GoToXY(PGMNAME_X, PGMNAME_Y);
                    Write(PgmFileName);
                    TextColor(CHARCOLOR);
                    Step           := 1;
                    Step2Show      := 1;
                    LearnLines     := 1;
                    UpDateScreen   := TRUE;
                  END;
              END;{NOT LearnMODE}

          END;{Extended Keys}

      {------------( Look for PF Keys )-------------------------}

        IF (ResponseType = PFKEY) THEN
          BEGIN
            UpdateScreen := TRUE;

            IF (KeyCode = F1 ) THEN
             BEGIN

              ClrScr;
              Writeln('********  P.R.CRANE  HELP SCREEN  *************  (Hit Any Key to RETURN)');
              Writeln('GENERAL INFO: P.R.CRANE is a computer-controlled crane built from an old IBM');
              Writeln('5152 Graphics Printer.  This software controls a P.R.CRANE with a standard');
              Writeln('Printer Port and Printer Cable.  NOTE: You must first tell the system which');
              Writeln('printer port you will be using, and which Text Editor you will be using.');
              Writeln('You should add the following two lines to your AUTOEXEC.BAT file, although  ');
              Writeln('you MAY type these lines at the DOS Prompt to try them out:                  ');
              Writeln('SET PARPORT=LPT1   (Or LPT2 or LPTM depending of which port you use.)        ');
              Writeln('SET EDITOR=C:\DOS\EDIT.EXE  (Or the location of your plain Text Editor.)    ');
              Writeln('NOTES: See the READMEPR.TXT file for more information.');
              Writeln('MOVEMENT is done with the 4 Arrow Keys. Insert and Delete turn magnet on & off.');
              Writeln('SCREEN SECTIONS: FKey functions shown at top. Upper right box shows current   ');
              Writeln('settings for movement speed (Steps per Second) and Movement size (Total steps)');
              Writeln('Change these settings with + and - keys. Use * and / keys to "bounce" select');
              Writeln('which value to change. Next below are status areas. Messages appear at bottom.');
              Writeln('Left Screen Area: Shows any active .CRL program. Current step is highlighted.  ');
              Writeln('F2 and F10 LOAD and SAVE stored .CRL programs. F5 / F6 RUN and STOP the program');
              Writeln('F4 temporarily stops and runs your Text Editor on the current .CRL program. ');
              Writeln('F7 Takes a Single Step in the .CRL Program. ALT-N: New   ALT-D: Pgm Description');
              Writeln('F9 Starts or Ends LEARN MODE. Each Movement made is saved as a program line.');
              Writeln('.CRL COMMANDS: UP DN RT LF movements are followed by a value. Magnet: MG 0 or 1');
              Writeln('UD and RL set the Steps/Second for Up/Down and Right/Left. WA WAITS (Value: Ms)');
              Writeln('LP LOOPS back to the beginning (Value times)                              ');
              Writeln('Questions?? EMAIL: tking@together.net UPDATES: homepages.together.net/~tking ');

              REPEAT UNTIL KeyPressed;
              ClrScr;
              InitScreen;
              UpdateScreen := TRUE
             END;

            IF (KeyCode = F2 ) AND (NOT LearnMODE) THEN
             BEGIN
               ShowMsg('LOAD PROGRAM');
               Delay(500);
               BlankMsg;
               PgmFileName := GetUserString('Filename = ', '',
                                            17,24);
               IF (POS('.', PgmFileName) = 0)
                 THEN PgmFileName := PgmFileName + '.crl';
               InitArrays;
               LoadPgm(PgmFileName);
               OldPgmFileName := PgmFileName;
             END;

            IF (KeyCode = F3 ) THEN
             BEGIN
               ShowMsgOpt('F3:Quit ??(Y/N)',LightRed,0);
               KbChar := ReadKey;
               BlankMsg;
               IF (KbChar = 'Y') OR (KbChar = 'y') THEN Done := TRUE;
             END;

            IF (KeyCode = F4 )  AND (NOT LearnMODE) THEN
             BEGIN
               ShowMsg('EDIT');
               ExecAndCheck(EditorPath, PgmFileName);
               InitArrays;
               InitScreen;
               LoadPgm(PgmFileName);
             END;

            IF (KeyCode = F5 )  AND (NOT LearnMODE) THEN
             BEGIN
               IF (BMState = ENDED) THEN Step := 1; {??}
    (*         IF (BMState = STOPPED) THEN Step := 1; {??}*)
               BMState := RUNNING;
             END;

            IF (KeyCode = F6 )  AND (NOT LearnMODE) THEN
             BEGIN
               BMState := STOPPED;
             END;

            IF (KeyCode = F7 )  AND (NOT LearnMODE) THEN
             BEGIN
               IF (BMState = ENDED) THEN
                 BEGIN
                   Step       := 1; {??}
                   Step2Show  := 1; {??}
                 END;
               BMState := STEPPING;
             END;

            IF (KeyCode = F8 )  AND (NOT LearnMODE) THEN
             BEGIN
               LoopCount := 1;
             END;

            IF (KeyCode = F9       ) THEN
              BEGIN
               ShowMsg('LEARN');
               IF LearnMODE
                 THEN
                   BEGIN
                     GoToXY(LEARN_X, LEARN_Y);
                     Write('LEARN MODE END');
                     PgmTopStep   := LearnLines -1;
                     Step         := 1;
                     Step2Show    := 1;
                     PgmFileName  := 'NEWPGM.CRL';
                     SavePgm(PgmFileName);
                     InitArrays;
                     LoadPgm(PgmFileName);
                     UpdateScreen := TRUE;
                     LearnMODE    := FALSE
                   END
                 ELSE
                   BEGIN
                     TextColor(LightRed);
                     GoToXY(LEARN_X, LEARN_Y);
                     Write('LEARN MODE !! ');
                     TextColor(CHARCOLOR);
                     LearnMODE  := TRUE;
                   END;
              END;
            IF (KeyCode = F10      ) THEN
              BEGIN
                IF NOT LearnMode THEN
                  BEGIN
                    BlankMsg;
                    PgmFileName := GetUserString('Filename = ', '',
                                                 17,24);
                    IF (PgmFileName[1] = '=') THEN
                       IF (Length(OldPgmFileName) > 4)
                         THEN PgmFileName := OldPgmFileName
                         ELSE PgmFileName := 'DEFAULT';
                    IF (POS('.', PgmFileName) = 0)
                      THEN PgmFileName := PgmFileName + '.crl';
                    OldPgmFileName := PgmFileName;
                    IF (Length(PgmFileName) >= 5) THEN
                      BEGIN
                        SavePgm(PgmFileName);
                        Msg := 'SAVED AS: ' + PgmFileName;
                        InitArrays;
                        LoadPgm(PgmFileName);
                        ShowMsg(Msg);
                        SaveMsg(Msg);
                      END;
                  END
                ELSE FailTone;
              END;

          END;{PFKeys}

{-------------------( 'REGULAR KEYS'         )--------------------------}
        IF (ResponseType = AlphaNumChar) THEN

          BEGIN
            UpDateScreen := TRUE;

          END;{AlphaNumChar}

      END;{Not NoKey}

    GoToXY(DefaultX,DefaultY);
  UNTIL Done;

{---------( END OF MAIN PROGRAM LOOP)------------------------------------}
  GoToXY(3,21);
  Delay(1000);
  Window(1,1,80,25);
  ClrScr;

  {---------( END OF  PROGRAM )------------------------------------}


END.{Main Program}

