{---------------------------------------------------------------------------}
{ BitMachine Program:  PRCRTEST                                             }
{ DATE:         02/12/99                                                    }
{ PROGRAMMER:   Terry King                                                  }
{ WHAT IT DOES: Tests for Rewired 5152 Graphics Printer as CRANE or WINCH   }
{                                                                           }
{ HISTORY:                                                                  }
{ 02/14/99   Updating to match PRCRANE                                      }
{---------------------------------------------------------------------------}
PROGRAM   PRCRTEST;

{---------------------------------------------------------------------------}
{               U S E S : WHAT OTHER PASCAL "UNITS" YOU WILL USE            }
{                                                                           }
{ NOTE: Other sets of tools or procedures that other people have created    }
{       that you want to re-use here.                                       }
{---------------------------------------------------------------------------}

USES
  CRT, DOS,
  CtrlBrk,                      { Ctrl-Break handling}
  HexUtil,
  ParPort;                      { Parallel Port Support }


{---------------------------------------------------------------------------}
{               T Y P E S        : FOR YOUR DESIGN                          }
{                                                                           }
{ NOTE: Here, you define data structures called TYPEs for your design       }
{---------------------------------------------------------------------------}
{TYPE}

{---------------------------------------------------------------------------}
{               C O N S T A N T S: FOR YOUR DESIGN                          }
{                                                                           }
{ NOTE: Here, you have to tell what is hooked up to the Digital In and      }
{       Digital Out Bits! Also other 'constants' for your design.           }
{---------------------------------------------------------------------------}
CONST

CHARCOLOR       = Yellow;   {Our screen colors}
BACKCOLOR       = Blue;

BLANKS        = '                               ';

UpDnMotorArray  : ARRAY[1..4] OF Byte = ($0C, $1C, $3C, $2C);
RtLfMotorArray  : ARRAY[1..4] OF Byte = ($40, $41, $43, $42);


MotorsHoldMask  = $4C;  {Pattern of Digital Out bits for low-power hold mode }
RLMotorMask     = $03;  { Bits that control motor phases }
UDMotorMask     = $30;

{-------( Digital IN Bits: What they are being used for:  )--- -------------}

FormFeedSw     =  $01;      { Switches on Printer Panel IF optional}
LineFeedSw     =  $02;      { Rewiring is done  }
PaperOutSw_    =  $04;
OnLineSw_      =  $20;
Pin_8B_1       =  $80;      { Uncomitted input pin: for WhatEver }

{-------( Digital OUT Bits: What they are being used for:  )------  }

                      {Left/Right (Printhead) Motor }
LRMotorA_B    = 0;    { Two Bits that control 4-step motor sequence }
LRMotorC_D    = 1;

_LRMotorEnab  = 2;    {- Enable 12 or 24V power to Left/Right Motor  }
_LRMotorHi    = 3;    {- 24V, + 12V  power: (PRCRANE Always uses 24V }

                      { Up/Down (Paper Feed) Motor}
UDMotorA_B    = 4;    { Two Bits that control 4-step motor sequence }
UDMotorC_D    = 5;

_UDMotorEnab  = 6;    {- Enable 24V power }

MagnetOn      = 7;    { For optional Electromagnet connected from + 24V }



{---------------------------------------------------------------------------}
{               V A R I A B L E S: FOR YOUR MACHINE DESIGN                  }
{                                                                           }
{ NOTE: Here, you have to tell what variables you will use to hold          }
{       temporary values for things. You can add lots more of these here.   }
{---------------------------------------------------------------------------}
VAR {MainLine}
    Version     : string;
    Option      : string;
    KeyByte     : Char;
    n           : integer;    { Integers are whole numbers up to 32,767 }

    Done        : Boolean;
    Run         : Boolean;
    Skip        : Boolean;
    Debug       : Boolean;

    IndexUpDn   : Word;   {Stepper motor position index: global}
    IndexRtLf   : Word;

    RLMotorBits   : Byte;  { Bits of last RL Position }
    UDMotorBits   : Byte;  { Bits of last UD Position }

    StartDelayMs  : Word;

    TestNumber       : Word;

    Xnow, Ynow       : Byte;                    {Cursor location}

    Bit              : Boolean;


(*Ŀ
   INTERNAL PROCEDURES and FUNCTIONS                                       
  *)

PROCEDURE MotorsHold;

VAR MotorsHoldData : Byte;

BEGIN
  RLMotorBits     := (RtLfMotorArray[IndexRtLf] AND RLMotorMask);
  UDMotorBits     := (UpDnMotorArray[IndexUpDn] AND UDMotorMask);
  MotorsHoldData  := MotorsHoldMask OR RLMotorBits OR UDMotorBits;
  WriteDo(MotorsHoldData);
  IF Debug THEN Writeln('HOLD: ', Byte2Bin(MotorsHoldData));
END;{MotorsHold}

{-----------------------------------------------------------------------}

PROCEDURE ShowAction(Action: String);
VAR SaveX, SaveY : Word;

  BEGIN
    SaveX := WhereX;
    SaveY := WhereY;
    GoToXY(1,3);
    Write(Action);
    GoToXY(SaveX, SaveY);
  END;

{-----------------------------------------------------------------------}
Procedure CraneUp(Steps: Word; StepDelayMs: Word);
{NOTE: Uses Global IndexUpDn }

BEGIN
  N := 0;
  ShowAction('MOVING UP');
  WriteDO(UpDnMotorArray[IndexUpDn] OR RLMotorBits); {Power up with last Pattern }
  Delay(StartDelayMs);
  REPEAT
    BEGIN
      INC(n);
      IF (IndexUpDn = 4) THEN IndexUpDn := 1 ELSE INC(IndexUpDn);
      WriteDO(UpDnMotorArray[IndexUpDn] OR RLMotorBits);
      IF Debug THEN Writeln('UP: ', Byte2Bin(UpDnMotorArray[IndexUpDn] OR RLMotorBits));
      Delay(StepDelayMs);
    END;
  UNTIL (n = Steps) OR BreakFlag;
  UDMotorBits := (UpDnMotorArray[IndexUpDn] AND UDMotorMask);
  MotorsHold;
  ShowAction(BLANKS);

END;{CraneUp}

{-----------------------------------------------------------------------}

Procedure CraneDn(Steps: Word; StepDelayMs: Word);
{NOTE: Uses Global IndexUpDn }

BEGIN

  N := 0;
  ShowAction('MOVING DOWN');
  WriteDO(UpDnMotorArray[IndexUpDn] OR RLMotorBits); {Power up with last Pattern }
  Delay(StartDelayMs);
  REPEAT
    BEGIN
      INC(n);
      IF (IndexUpDn = 1) THEN IndexUpDn := 4 ELSE DEC(IndexUpDn);
      WriteDO(UpDnMotorArray[IndexUpDn] OR RLMotorBits);
      IF Debug THEN Writeln('DN: ', Byte2Bin(UpDnMotorArray[IndexUpDn] OR RLMotorBits));
      Delay(StepDelayMs);
    END;
  UNTIL (n = Steps) OR BreakFlag;
  MotorsHold;
  ShowAction(BLANKS);

END;{CraneDn}

{-----------------------------------------------------------------------}

Procedure CraneLf(Steps: Word; StepDelayMs: Word);
{NOTE: Uses Global IndexRtLf }

BEGIN

  N := 0;
  ShowAction('MOVING LEFT');
  WriteDO(RtLfMotorArray[IndexRtLf] OR UDMotorBits); {Power up with last Pattern }
  Delay(StartDelayMs);
  REPEAT
    BEGIN
      INC(n);
      IF (IndexRtLf = 4) THEN IndexRtLf := 1 ELSE INC(IndexRtLf);
      WriteDO(RtLfMotorArray[IndexRtLf] OR UDMotorBits);
      IF Debug THEN Writeln('LF: ', Byte2Bin(RtLfMotorArray[IndexRtLf] OR UDMotorBits));
      Delay(StepDelayMs);
    END;
  UNTIL (n = Steps) OR BreakFlag;
  MotorsHold;
  ShowAction(BLANKS);

END;{CraneLf}

{-----------------------------------------------------------------------}

Procedure CraneRt(Steps: Word; StepDelayMs: Word);
{NOTE: Uses Global IndexRtLf }

BEGIN

  N := 0;
  ShowAction('MOVING RIGHT');
  WriteDO(RtLfMotorArray[IndexRtLf] OR UDMotorBits); {Power up with last Pattern }
  Delay(StartDelayMs);
  REPEAT
    BEGIN
      INC(n);
      IF (IndexRtLf = 1) THEN IndexRtLf := 4 ELSE DEC(IndexRtLf);
      WriteDO(RtLfMotorArray[IndexRtLf] OR UDMotorBits);
      IF Debug THEN Writeln('RT: ', Byte2Bin(RtLfMotorArray[IndexRtLf] OR UDMotorBits));
      Delay(StepDelayMs);
    END;
  UNTIL (n = Steps) OR BreakFlag;
  MotorsHold;
  ShowAction(BLANKS);

END;{CraneRt}

{-----------------------------------------------------------------------}
Procedure CheckUserKey;
    BEGIN
      Write(' ACTION (Run/Next/Back/Quit) ? ');
      KeyByte := ReadKey;
      Writeln;
      Run := FALSE;
      IF (KeyByte = 'q') OR (KeyByte = 'Q') THEN Done     := TRUE;
      IF (KeyByte = 'r') OR (KeyByte = 'R') THEN Run      := TRUE;
      IF (KeyByte = 'n') OR (KeyByte = 'N') THEN Inc(TestNumber);
      IF (KeyByte = 'b') OR (KeyByte = 'B') THEN Dec(TestNumber);
    END;{CheckUserKey}


(*Ŀ
   MAIN PROGRAM:                                                           
  *)
BEGIN

 Debug      := FALSE;
 Version    := '1.18  ( Test 4 Rev     )02/24/99';

 BreakFlag  := FALSE;  {Set up to allow Ctrl-Break to exit tight loops }
 SetCtrlBrk;           { Install and Enable interrupt handler }

 Debug      := FALSE;
 Done       := FALSE;
 Run        := FALSE;
 Skip       := FALSE;

 WriteDO(MotorsHoldMask);  { Set motor control bits to low-power hold mode}

 IndexUpDn  := 1;
 IndexRtLf  := 1;
 StartDelayMs := 100;
 RLMotorBits  := 0;
 UDMotorBits  := 0;

 TestNumber := 1;

 TextBackGround(BACKCOLOR);
 TextColor(CHARCOLOR);
 ClrScr;
 GoToXY(1,1);
 Writeln('  ---- TEST PROGRAM FOR REWIRED IBM 5152 PRINTER FOR "P.R.CRANE " ---- ');
 writeln('                       ',version);

 GoToXY(1,4);
 Write('Push Q to Quit     <Ctrl> and Break to Abort a move and stop immediately ');

 GoToXY(1,8);
 REPEAT
   BreakFlag := FALSE;    {OK: Run unless Ctrl-Break is pushed }
   delay(200);
   IF NOT Done THEN
    BEGIN
      Write('Test ', TestNumber :2, ' ');
      n := 0;
 {*******************************************************************}
    CASE TestNumber OF

     1:
      BEGIN
        Write('Crane Up / Down SLOW Test ');
        CheckUserKey;
        IF Run THEN
          BEGIN
            Delay(1000);
            Writeln('Crane Up SLOW');
            CraneUp(10, 200);
            Delay(500);
            Writeln('Crane DownSLOW');
            CraneDn(10, 200);
          END;{Continue}
      END;

 {*******************************************************************}
     2:
      BEGIN
        Write('Crane Up / Down Test  ');
        CheckUserKey;
        IF Run THEN
          BEGIN
            Delay(1000);
            Writeln('Crane Up');
            CraneUp(100, 8);
            Delay(500);
            Writeln('Crane Down');
            CraneDn(100, 3);
          END;
      END;

 {*******************************************************************}
     3:
      BEGIN
        Write('Crane RT / Left Slow Test  ');
        CheckUserKey;
        IF Run THEN
          BEGIN
            Delay(1000);
            Writeln('Crane Rt SLOW');
            CraneRt(10, 200);
            Delay(500);
            Writeln('Crane Lf SLOW');
            CraneLf(10, 300);
          END;
      END;

 {*******************************************************************}
     4:
      BEGIN
        Write('Crane Rt / Left Test  ');
        CheckUserKey;
        IF Run THEN
          BEGIN
            Writeln('Test 4');
            Delay(1000);
            Writeln('Crane Right');
            CraneRt(100, 20);
            Delay(500);
            WriteLn('Crane Left');
            CraneLf(100, 10);
          END;
      END;


 {*******************************************************************}
     5:
      BEGIN
        Write('Multi-Function Demo   ');
        CheckUserKey;
        IF Run THEN
          BEGIN
            Delay(1000);
            Writeln('Crane Right');
            CraneRt(200, 20);
            WriteLn('Crane Left');
            CraneLf(200, 8);
            Delay(500);
            Writeln('Crane Right');
            CraneRt(200, 5);
            WriteLn('Crane Left');
            CraneLf(200, 3);

            Delay(1000);
            Writeln('Crane Up');
            CraneUp(200, 40);
            Writeln('Crane Down');
            CraneDn(200, 24);
            Delay(500);
            Writeln('Crane Up');
            CraneUp(400, 5);
            Writeln('Crane Down');
            CraneDn(400, 3);
          END;
      END;

 {*******************************************************************}
     6:
      BEGIN
        Write('Test 6');
        CheckUserKey;
      END;

 {*******************************************************************}
     7:
      BEGIN
        Writeln('ALL DONE !! ');
        delay(2000);
        Done := TRUE;
      END;

 {*******************************************************************}

    END;{CASE}

    END;{IF Not Done}

 UNTIL Done;

 ResetCtrlBrk;                { Clean stuff up and quit }

{
 Window(1,1,80,25);
 ClrScr;
}
 {---------( END OF  PROGRAM )------------------------------------}


END.{Main Program}

